import logging
from os import path

from src.ansidocs.classes.layout import Layout
from src.ansidocs.classes.project_parts.meta import Meta
from src.ansidocs.classes.project_parts.playbooks import Playbooks   # noqa: F401
from src.ansidocs.classes.project_parts.plugins import Plugins   # noqa: F401
from src.ansidocs.classes.project_parts.defaults import Defaults   # noqa: F401
from src.ansidocs.classes.project_parts.roles import Roles   # noqa: F401


logger = logging.getLogger(__name__)


class Project():
    def __init__(self, layout: Layout, root_dir: str, namespace: str = None):
        self.root_dir = path.abspath(root_dir)
        if not path.exists(self.root_dir):
            raise FileNotFoundError(f"Unable to find project directory {self.root_dir}")

        abs_doc_dir = path.abspath(path.join(self.root_dir, layout.docs))
        if not path.exists(abs_doc_dir):
            logger.warning(f"Unable to find documentation directory in project, {abs_doc_dir}")
            self.docs_dir = None
        else:
            self.docs_dir = abs_doc_dir

        self.meta = Meta(layout=layout.meta, project_root=self.root_dir, namespace=namespace)
        self.layout = layout
        self.parts = self.__find_parts(layout.part_paths)
        self.readme_file = path.join(self.root_dir, layout.readme)
        self._description = None
        self._usage = None

    def __find_parts(self, part_paths: dict):
        results = []
        for k, v in part_paths.items():
            logger.debug(f"Looking for part {k=} and path {v=}")
            try:
                part_class = globals()[str(k).capitalize()]
                results += [part_class(project_root=self.root_dir, layout=self.layout)]
                logger.info(f"Loaded project part {k}")
            except KeyError:
                raise Exception(f"This program does not support project part '{k}'"
                                f" from the layout configuration '{self.layout.name}'")

        return set(results)

    @property
    def description(self):
        if self._description:
            return self._description

        if self.docs_dir:
            logger.debug("Looking for a custom description.md file for this project")
            try:
                with open(path.join(self.docs_dir, "description.md")) as f:
                    self._description = f.read()
            except FileNotFoundError:
                logger.debug("Unable to find a description.md in docs directory. Using meta description as fallback")
        else:
            logger.debug("No docs directory. Using meta description as fallback")

        self._description = self.meta.description
        return self._description

    @description.setter
    def description(self, description):
        self._description = description

    @property
    def usage(self):
        if self._usage:
            return self._usage

        if self.docs_dir:
            logger.debug("Looking for a custom usage.md file for this project")
            try:
                with open(path.join(self.docs_dir, "usage.md")) as f:
                    self._usage = f.read()
            except FileNotFoundError:
                logger.info("Unable to find a custom usage.md in docs directory")
        else:
            logger.debug("No docs directory.No custom usage markdown will be added")

        self._usage = None
        return self._usage



from abc import ABC
from abc import abstractmethod
from typing import Iterable
from dataclasses import dataclass
from dataclasses import field
from os import path

from src.ansidocs.classes.layout import Layout


@dataclass
class AbstractDirContent(ABC):
    subdirs: Iterable
    parse_as_project: bool = field(default=False, init=False)


class AbstractProjectPart(ABC):
    def __init__(self, project_root: str, layout: Layout):
        self.project_root = path.abspath(project_root)
        self.layout = layout

    @abstractmethod
    def get_content():
        pass

    @abstractmethod
    def to_markdown() -> Iterable:
        pass

    def link_project(self, project: object):
        self.project = project



import logging
from os import path
import jinja2

from src.ansidocs.classes.project import Project


logger = logging.getLogger(__name__)


class ReadmeWriter():
    __jinja_env = {}

    def __init__(self, config_dir: str = None):
        if config_dir:
            self.load_templates(config_dir)

    def load_templates(self, config_dir: str):
        loader = jinja2.FileSystemLoader(path.join(config_dir, 'templates'), encoding='utf8')
        ReadmeWriter.__jinja_env = jinja2.Environment(loader=loader, autoescape=True)
        ReadmeWriter.__jinja_env.filters['basename'] = basename

    @staticmethod
    def jinja_env():
        return ReadmeWriter.__jinja_env

    @staticmethod
    def update(project: Project):
        logger.info(f"Updating readme for project, {project.readme_file}")
        try:
            readme_template = ReadmeWriter.__jinja_env.get_template(f'{project.layout.name}/README.md.j2')
        except jinja2.exceptions.TemplateNotFound:
            logger.debug(f"No custom readme template for layout '{project.layout.name}'")
            readme_template = ReadmeWriter.__jinja_env.get_template('defaults/README.md.j2')

        description_block = ReadmeWriter.__render_description(project=project)
        usage_block = ReadmeWriter.__render_usage(project=project)
        footer_block = ReadmeWriter.__render_footer(project=project)

        readme_text = readme_template.render(
            description_block=description_block,
            usage_block=usage_block,
            footer_block=footer_block
        )
        logger.info("Writing data to readme")
        with open(path.join(project.root_dir, project.readme_file), 'w') as file:
            file.write(readme_text)

    @staticmethod
    def __render_description(project: Project):
        logger.info("Compiling description section of readme for project")
        try:
            template = ReadmeWriter.__jinja_env.get_template(f'{project.layout.name}/description.md.j2')
        except jinja2.exceptions.TemplateNotFound:
            logger.debug(f"No custom description readme template for layout '{project.layout.name}'")
            template = ReadmeWriter.__jinja_env.get_template('defaults/description.md.j2')
        return template.render(project=project)

    @staticmethod
    def __render_usage(project: Project):
        logger.info("Compiling usage section of readme for project")
        try:
            template = ReadmeWriter.__jinja_env.get_template(f'{project.layout.name}/usage.md.j2')
        except jinja2.exceptions.TemplateNotFound:
            logger.debug(f"No custom usage readme template for layout '{project.layout.name}'")
            template = ReadmeWriter.__jinja_env.get_template('defaults/usage.md.j2')
        return template.render(project=project)

    @staticmethod
    def __render_footer(project: Project):
        logger.info("Compiling footer section of readme for project")
        try:
            template = ReadmeWriter.__jinja_env.get_template(f'{project.layout.name}/footer.md.j2')
        except jinja2.exceptions.TemplateNotFound:
            logger.debug(f"No custom footer readme template for layout '{project.layout.name}'")
            template = ReadmeWriter.__jinja_env.get_template('defaults/footer.md.j2')
        return template.render(project=project)


def basename(path):
    return path.basename(path)



from os import path
from os import listdir
import logging
from dataclasses import dataclass

from src.ansidocs.classes.abstract_project_part import AbstractProjectPart
from src.ansidocs.classes.abstract_project_part import AbstractDirContent
from src.ansidocs.classes.layout import Layout


logger = logging.getLogger(__name__)


@dataclass
class PlaybookDirContent(AbstractDirContent):
    """Class for keeping track of stuff in a playbooks directory."""
    playbooks: list
    subdirs: dict


class Playbooks(AbstractProjectPart):
    def __init__(self, project_root: str, layout: Layout):
        super().__init__(layout=layout, project_root=project_root)
        self.root_dir = path.abspath(path.join(self.project_root, layout.part_paths['playbooks']))
        if not path.exists(self.root_dir):
            raise FileNotFoundError(f"Unable to find playbooks project part directory {self.root_dir}")

    def get_content(self, search_path: str = None):
        playbooks = []
        subdirs = {}
        ignore_dirs = ["templates", "files", "inventories", "group_vars"]
        if not search_path:
            search_path = self.root_dir

        for fsobj in listdir(search_path):
            obj_path = path.join(search_path, fsobj)
            if path.isdir(obj_path):
                if fsobj in ignore_dirs:
                    continue
                else:
                    nested_struct = self.get_content(obj_path)
                    subdirs[fsobj] = nested_struct
            else:
                if fsobj.endswith(".yml") or fsobj.endswith(".yaml"):
                    playbooks += [fsobj]
        return PlaybookDirContent(playbooks=playbooks, subdirs=subdirs)

    def to_markdown(self):
        relative_path = self.layout.part_paths['playbooks']
        markdown = ["### Playbooks", ""]
        content = self.get_content()

        for subdir_name, subdir_content in content.subdirs.items():
            markdown += self.__subdir_to_markdown(subdir_name, subdir_content)

        if content.playbooks:
            markdown += ["#### Other Plays", ""]
            for playbook in content.playbooks:
                markdown += [f"- [{playbook}]({path.join(relative_path, playbook)})"]
            markdown += [""]

        if len(markdown) == 2:   # header only
            markdown += ["There are no playbooks in the project."]
        else:
            markdown.insert(2, "The following playbooks are available to use from this project:")
            markdown.insert(3, "")
        return markdown

    def __subdir_to_markdown(self, subdir_name: str, subdir: PlaybookDirContent):
        relative_path = self.layout.part_paths['playbooks']
        markdown = [f"#### {str(subdir_name).capitalize()}", ""]
        if subdir.playbooks:
            for playbook in subdir.playbooks:
                markdown += [f"- [{playbook}]({path.join(relative_path, subdir_name, playbook)})"]
            markdown += [""]
        for child_name, child_subdir in subdir.subdirs.items():
            markdown += self.__subdir_to_markdown(f"{subdir_name}/{child_name}", child_subdir)

        if len(markdown) == 2:   # header only
            return []
        else:
            return markdown



from dataclasses import dataclass
from dataclasses import field


@dataclass
class MetaLayout:
    file: str
    name_attr: str = field(default_factory=lambda: "name")
    nested_key: str = field(default_factory=lambda: None)


@dataclass
class Layout:
    """Class for describing a project type layout."""
    name: str
    meta: MetaLayout
    readme: str = field(default_factory=lambda: "./README.md")
    docs: str = field(default_factory=lambda: "./docs")
    part_paths: dict = field(default_factory=lambda: {})
    part_namespace: str = field(default_factory=lambda: None)

    def __post_init__(self):
        self.meta = MetaLayout(
            file=self.meta['file'],
            name_attr=self.meta.get('name_attr', 'name'),
            nested_key=self.meta.get('nested_key', '')
        )



import logging
from os import path
import yaml

from src.ansidocs.classes.layout import MetaLayout


logger = logging.getLogger(__name__)


class Meta:
    def __init__(self, layout: MetaLayout, project_root: str, namespace: str = None):
        super().__init__()
        self.description = ""
        self.layout = layout
        meta_abs = path.abspath(path.join(project_root, layout.file))
        if not path.exists(meta_abs):
            raise FileNotFoundError(f"Unable to find meta file {meta_abs}")

        with open(meta_abs, "r") as f:
            meta_data = yaml.safe_load(f)

        if layout.nested_key:
            self.__dict__.update(meta_data[layout.nested_key])
        else:
            self.__dict__.update(meta_data)

        try:
            self.name = self.__dict__[layout.name_attr]
        except KeyError:
            pass

        if namespace:
            self.namespace = namespace

        try:
            self.qualified_name = ".".join([self.namespace, self.name])
        except AttributeError as e:
            logger.debug(f"Missing piece of qualified name from meta file, {e}")
            self.qualified_name = None



from os import path
import logging
from dataclasses import dataclass
import glob

from src.ansidocs.classes.abstract_project_part import AbstractProjectPart
from src.ansidocs.classes.abstract_project_part import AbstractDirContent
from src.ansidocs.classes.layout import Layout


logger = logging.getLogger(__name__)


@dataclass
class RolesDirContent(AbstractDirContent):
    """Class for keeping track of stuff in a plugin directory."""
    pass


class Roles(AbstractProjectPart):
    def __init__(self, project_root: str, layout: Layout):
        super().__init__(project_root=project_root, layout=layout)
        self.root_dir = path.abspath(path.join(self.project_root, layout.part_paths['roles']))
        if not path.exists(self.root_dir):
            raise FileNotFoundError(f"Unable to find Defaults project part directory {self.root_dir}")

    def get_content(self):
        content = []
        subfiles = glob.glob(f'{self.root_dir}/*')
        content = filter(lambda f: path.isdir(f), subfiles)
        ret = RolesDirContent(subdirs=content)
        ret.parse_as_project = True
        return ret

    def to_markdown(self):
        relative_path = self.layout.part_paths['roles']
        markdown = ["### Roles", ""]
        content = self.get_content()

        if content.subdirs:
            markdown += ["The following roles are available for use:", ""]
            for role in content.subdirs:
                role_name = path.basename(role)
                markdown += [f"- [{role_name}]({path.join(relative_path, role_name)})"]
        else:
            markdown += ["There are no roles availble for this project."]
        return markdown



from src.ansidocs.classes.static.config import Config


class UknownLayoutException(Exception):
    """Exception raised when a project does not match a known layout

    Attributes:
        known_layouts     -- layouts that we do know about
        project_directory -- path to the project with unknown layout
    """

    def __init__(self, project_directory):
        self.known_layouts = Config.layouts()
        self.project_directory = project_directory
        super().__init__(f"Project at {project_directory} does not match any of the known layouts.")



import logging
import os
import shutil
import pathlib

from src.ansidocs.classes.project import Project
from src.ansidocs.classes.static.config import Config
from src.ansidocs.classes.static.readme_writer import ReadmeWriter
from src.ansidocs.classes.exceptions import UknownLayoutException


logger = logging.getLogger(__name__)


def determine_project_layout(project_path: str):
    full_dir = os.path.abspath(project_path)
    if not os.path.exists(full_dir):
        raise FileNotFoundError(f"Unable to find any files at {project_path}")

    for layout in Config.layouts():
        if os.path.exists(os.path.join(full_dir, layout.meta.file)):
            return layout
    else:
        return None


def process_directory(project_path: str, namespace: str = None):
    layout = determine_project_layout(project_path=project_path)
    if not layout:
        raise UknownLayoutException(project_directory=project_path)

    project = Project(root_dir=project_path, layout=layout, namespace=namespace)
    if layout.part_namespace and layout.part_namespace != 'none':
        try:
            part_namespace = getattr(project.meta, layout.part_namespace)
        except AttributeError:
            logger.critical(f"Unrecognized attribute '{layout.part_namespace}' "
                            "on parent project. Cant set part namespace.")
            raise
    else:
        part_namespace = None

    for part in project.parts:
        content = part.get_content()
        if content.parse_as_project:
            for subdir in content.subdirs:
                process_directory(project_path=subdir, namespace=part_namespace)

    ReadmeWriter.update(project)


def init_config_dir(config_dir: str, refresh_configs: bool = False):
    if refresh_configs:
        logger.warning("Config refresh requested. Config directory will be removed and recreated")
        try:
            shutil.rmtree(config_dir)
        except FileNotFoundError:
            pass
    if not os.path.exists(config_dir):
        logger.info("No config directory exists, creating one with defaults now")
        current_file_path = pathlib.Path(__file__).parent.resolve()
        shutil.copytree(f"{current_file_path}/config", config_dir)
        os.chmod(config_dir, 0o755)
        for root, dirs, files in os.walk(config_dir):
            for dir in dirs:
                os.chmod(os.path.join(root, dir), 0o755)
            for file in files:
                os.chmod(os.path.join(root, file), 0o644)
    else:
        logger.info("Config directory exists")


def generate(directory: str, config_dir: str = "ansidocs/config"):
    directory = directory.rstrip('/')
    logger.debug(f"{directory=}")
    logger.debug(f"{config_dir=}")
    init_config_dir()

    Config().load_file(config_dir)
    ReadmeWriter().load_templates(config_dir)
    try:
        process_directory(project_path=directory)
    except Exception as e:
        logger.critical(e)
        logger.critical("Caught fatal exception and exiting. See output above")
    else:
        logger.info("Done, finished updating readmes successfully")


if __name__ == "__main__":
    logging.basicConfig(level="DEBUG")
    project_root = "/tmp/test"
    generate(directory=project_root)



import src.ansidocs.main as ansidocs
import argparse
from os import path
import logging


def parseArgs():
    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--loglevel", dest="loglevel", default='WARNING',
                        help="Log level for python logging, defaults " +
                        "to WARNING. Set to WARNING, INFO, DEBUG, ERROR")
    parser.add_argument("-c", "--config-dir", dest="config_dir",
                        help="Path to the config dir",
                        default=path.join(path.expanduser('~'), '.ansidocs'))

    subparsers = parser.add_subparsers(title='subcommands',
                                       description='valid subcommands',
                                       help='additional help')
    gen_parser = subparsers.add_parser("generate", aliases=["gen"], help="Generate or update an existing readme")
    gen_parser.add_argument("-d", "--directory", dest="directory", default=".",
                            help="Directory where readme should be generated")
    test_parser = subparsers.add_parser("test", help="Test if the existing readme matches the expected readme")

    conf_parser = subparsers.add_parser("config", help="Create or refresh your config directory")
    conf_parser.add_argument("-r", "--refresh-configs", dest="refresh_configs",
                             help="If flag is set, config dir will be removed and recreated with defaults",
                             action='store_true')

    gen_parser.set_defaults(func=gen)
    conf_parser.set_defaults(func=conf)
    test_parser.set_defaults(func=test)

    return parser.parse_args()


def main():
    args = parseArgs()
    logging.basicConfig(level=args.loglevel)
    args.func(args)


def gen(args):
    ansidocs.generate(
        directory=args.directory,
        config_dir=args.config_dir)


def conf(args):
    ansidocs.init_config_dir(
        config_dir=args.config_dir,
        refresh_configs=args.refresh_configs)


def test():
    pass



import os
import yaml
import logging

from src.ansidocs.classes.layout import Layout


logger = logging.getLogger(__name__)


class Config():
    __raw_conf = {}
    __layouts = []

    def __init__(self, config_dir: str = None):
        if config_dir:
            self.load_file(config_dir)

    def load_file(self, config_dir: str = ""):
        config_path = os.getenv(
            "ANSIDOCS_CONFIG",
            os.path.abspath(os.path.join(config_dir, "ansidocs.yml"))
        )
        with open(config_path) as f:
            Config.__raw_conf = yaml.safe_load(f)
        self.__load_layouts()

    def __load_layouts(self):
        project_layouts = []
        for k, v in Config.__raw_conf.get('project_layouts', {}).items():
            project_layouts += [Layout(name=k, **v)]

        Config.__layouts = project_layouts

    @staticmethod
    def raw_config(name: str, default: None):
        return Config.__raw_conf.get(name, default)

    @staticmethod
    def layouts():
        logger.debug(Config.__layouts)
        return Config.__layouts



from os import path
from dataclasses import dataclass
import yaml
import logging

from src.ansidocs.classes.abstract_project_part import AbstractProjectPart
from src.ansidocs.classes.abstract_project_part import AbstractDirContent
from src.ansidocs.classes.layout import Layout


logger = logging.getLogger(__name__)


@dataclass
class DefaultsDirContent(AbstractDirContent):
    """Class for keeping track of stuff in a plugin directory."""
    default_parameters: list


@dataclass
class DefaultParameter():
    """Class for keeping track of stuff in a plugin directory."""
    name: str
    value: object
    description: str
    required: bool


class Defaults(AbstractProjectPart):
    def __init__(self, project_root: str, layout: Layout):
        super().__init__(layout=layout, project_root=project_root)
        self.root_dir = path.abspath(path.join(self.project_root, layout.part_paths['defaults']))
        if not path.exists(self.root_dir):
            raise FileNotFoundError(f"Unable to find defaults project part directory {self.root_dir}")

    def get_values(self):
        with open(path.join(self.root_dir, "main.yml")) as f:
            defaults = yaml.safe_load(f)

        return defaults

    def get_descriptions(self):
        defaults_desc_file = path.join(self.project_root, self.layout.docs, "defaults.yml")
        try:
            with open(defaults_desc_file) as f:
                descr = yaml.safe_load(f)
        except FileNotFoundError:
            logger.critical(f"No defaults description file found at {defaults_desc_file}. "
                            "This file is required when defaults are present")
            raise
        return descr

    def get_content(self):
        content = []
        values = self.get_values()
        if values:
            descr = self.get_descriptions()

            for k, v in values.items():
                content += [DefaultParameter(
                    name=k,
                    value=v,
                    required=bool(v == ''),
                    description=descr.get(k, None)
                )]

        return DefaultsDirContent(subdirs=[], default_parameters=content)

    def to_markdown(self):
        markdown = ["### Variables", ""]
        content = self.get_content()

        if content.default_parameters:
            markdown += ["The following table contains variables you can override and their default values:"]
            markdown += ["<table>", "<tr>", "<th>Variable Name</th>", "<th>Required</th>"]
            markdown += ["<th>Default Value</th>", "<th>Description</th>", "</tr>"]
            for parameter in content.default_parameters:
                markdown += ["<tr>"]
                markdown += [
                    f"<th>{parameter.name}</th>",
                    f"<th>{parameter.required}</th>",
                    f"<th>{parameter.value}</th>",
                    f"<th>{parameter.description}</th>"
                ]
                markdown += ["</tr>"]
            markdown += ["</table>"]
        else:
            markdown += ["There are no input parameters availble for this project."]

        return markdown
